from flask_wtf import FlaskForm
from wtforms import (StringField, PasswordField, BooleanField, SubmitField,
                     ValidationError)
from wtforms.validators import DataRequired, Length, Email, Regexp, EqualTo
from ..models import User


class LoginForm(FlaskForm):
    login = StringField('Login', validators=[DataRequired(), Length(1, 64)])
    password = PasswordField('Password', validators=[DataRequired()])
    remember_me = BooleanField('Keep me logged in')
    submit = SubmitField('Log In')


class RegistrationForm(FlaskForm):
    email = StringField('Email', validators=[DataRequired(), Email()])
    username = StringField(
        'Username',
        validators=[DataRequired(),
                    Length(1, 64),
                    Regexp('^[A-Za-z][A-Za-z0-9_.]*$',
                           0,
                           'Usernames must have only letters, numbers, dots '
                           'or underscores')]
    )
    password = PasswordField(
        'Password',
        validators=[DataRequired(),
                    EqualTo('password2', message='Passwords must match.')]
    )
    password2 = PasswordField('Confirm password', validators=[DataRequired()])
    submit = SubmitField('Register')

    def validate_email(self, field):
        if User.query.filter_by(email=field.data.lower()).first():
            raise ValidationError('Email already registered.')

    def validate_username(self, field):
        if User.query.filter_by(username=field.data).first():
            raise ValidationError('Username already in use.')


class PasswordResetForm(FlaskForm):
    password = PasswordField(
        'New Password',
        validators=[
            DataRequired(),
            EqualTo('password2', message='Passwords must match')
        ]
    )
    password2 = PasswordField(
        'Confirm password',
        validators=[
            DataRequired(),
            EqualTo('password', message='Passwords must match.')
        ]
    )
    submit = SubmitField('Reset Password')


class PasswordResetRequestForm(FlaskForm):
    email = StringField('Email', validators=[DataRequired(), Email()])
    submit = SubmitField('Reset Password')


class ChangePasswordForm(FlaskForm):
    """
    Form to change information of currently logged in User. User can change
    informations about him on his own.
    """
    old_password = PasswordField('Old password', validators=[DataRequired()])
    new_password = PasswordField(
        'New password',
        validators=[
            DataRequired(),
            EqualTo('new_password2', message='Passwords must match.')
        ]
    )
    new_password2 = PasswordField(
        'Confirm new password',
        validators=[
            DataRequired(),
            EqualTo('new_password', message='Passwords must match.')
        ]
    )
    submit = SubmitField('Update Password')


class EditProfileForm(FlaskForm):
    email = StringField('Change Email',
                        validators=[Length(0, 254), DataRequired()])
    submit = SubmitField('Change Email')

    def __init__(self, user, *args, **kwargs):
        super(EditProfileForm, self).__init__(*args, **kwargs)
        self.user = user

    def validate_email(self, field):
        if field.data != self.user.email and \
                User.query.filter_by(email=field.data).first():
            raise ValidationError('Email already registered!')
